"""
Class object for LyricLocation
Documentation: https://github.com/timmo001/aiolyric

Generated by generator/generator.py - 2020-08-31 13:55:49.220121
"""
from typing import List
from .base import LyricBase, LyricBaseClient
from .device import LyricDevice


class Locationrolemapping(LyricBase):
    @property
    def locationID(self):
        return self.attributes.get("locationID", None)

    @property
    def role(self):
        return self.attributes.get("role", None)

    @property
    def locationName(self):
        return self.attributes.get("locationName", None)

    @property
    def status(self):
        return self.attributes.get("status", None)


class Users(LyricBase):
    @property
    def userID(self):
        return self.attributes.get("userID", None)

    @property
    def username(self):
        return self.attributes.get("username", None)

    @property
    def firstname(self):
        return self.attributes.get("firstname", None)

    @property
    def lastname(self):
        return self.attributes.get("lastname", None)

    @property
    def created(self):
        return self.attributes.get("created", None)

    @property
    def deleted(self):
        return self.attributes.get("deleted", None)

    @property
    def activated(self):
        return self.attributes.get("activated", True)

    @property
    def connectedHomeAccountExists(self):
        return self.attributes.get("connectedHomeAccountExists", True)

    @property
    def locationRoleMapping(self):
        return [
            Locationrolemapping(x)
            for x in self.attributes.get("locationRoleMapping", [])
        ]

    @property
    def isOptOut(self):
        return self.attributes.get("isOptOut", None)

    @property
    def isCurrentUser(self):
        return self.attributes.get("isCurrentUser", True)


class Time(LyricBase):
    @property
    def start(self):
        return self.attributes.get("start", None)

    @property
    def end(self):
        return self.attributes.get("end", None)


class Schedules(LyricBase):
    @property
    def time(self):
        return [Time(x) for x in self.attributes.get("time", [])]

    @property
    def days(self):
        return self.attributes.get("days", [])


class ConfigurationFacerecognition(LyricBase):
    @property
    def enabled(self):
        return self.attributes.get("enabled", False)

    @property
    def maxPersons(self):
        return self.attributes.get("maxPersons", None)

    @property
    def maxEtas(self):
        return self.attributes.get("maxEtas", None)

    @property
    def maxEtaPersons(self):
        return self.attributes.get("maxEtaPersons", None)

    @property
    def schedules(self):
        return [Schedules(x) for x in self.attributes.get("schedules", [])]


class Configuration(LyricBase):
    @property
    def faceRecognition(self):
        return ConfigurationFacerecognition(self.attributes.get("faceRecognition", {}))


class LyricLocation(LyricBaseClient):
    @property
    def locationID(self):
        return self.attributes.get("locationID", None)

    @property
    def name(self):
        return self.attributes.get("name", None)

    @property
    def country(self):
        return self.attributes.get("country", None)

    @property
    def zipcode(self):
        return self.attributes.get("zipcode", None)

    @property
    def devices(self) -> List[LyricDevice]:
        devices = []
        for x in self.attributes.get("devices", []):
            x["locationID"] = self.locationID
            devices.append(LyricDevice(self.client, x))
        return devices

    @property
    def devices_dict(self) -> dict:
        devices_dict: dict = {}
        for device in self.devices:
            devices_dict[device.macID] = device
        return devices_dict

    @property
    def users(self):
        return [Users(x) for x in self.attributes.get("users", [])]

    @property
    def timeZone(self):
        return self.attributes.get("timeZone", None)

    @property
    def ianaTimeZone(self):
        return self.attributes.get("ianaTimeZone", None)

    @property
    def daylightSavingTimeEnabled(self):
        return self.attributes.get("daylightSavingTimeEnabled", True)

    @property
    def geoFenceEnabled(self):
        return self.attributes.get("geoFenceEnabled", False)

    @property
    def predictiveAIREnabled(self):
        return self.attributes.get("predictiveAIREnabled", False)

    @property
    def comfortLevel(self):
        return self.attributes.get("comfortLevel", None)

    @property
    def geoFenceNotificationEnabled(self):
        return self.attributes.get("geoFenceNotificationEnabled", False)

    @property
    def geoFenceNotificationTypeId(self):
        return self.attributes.get("geoFenceNotificationTypeId", None)

    @property
    def configuration(self):
        return Configuration(self.attributes.get("configuration", {}))



"""Lyric: Exceptions"""


class LyricException(BaseException):
    """Raise this when something is off."""


class LyricAuthenticationException(LyricException):
    """Raise this when there is an authentication issue."""



"""
Class object for LyricDevice
Documentation: https://github.com/timmo001/aiolyric

Generated by generator/generator.py - 2020-08-31 14:06:02.854691
"""
from .base import LyricBase, LyricBaseClient


class Vacationhold(LyricBase):
    @property
    def enabled(self):
        return self.attributes.get("enabled", False)


class Currentscheduleperiod(LyricBase):
    @property
    def day(self):
        return self.attributes.get("day", None)

    @property
    def period(self):
        return self.attributes.get("period", None)


class Schedulecapabilities(LyricBase):
    @property
    def availableScheduleTypes(self):
        return self.attributes.get("availableScheduleTypes", [])

    @property
    def schedulableFan(self):
        return self.attributes.get("schedulableFan", False)


class Scheduletype(LyricBase):
    @property
    def scheduleType(self):
        return self.attributes.get("scheduleType", None)

    @property
    def scheduleSubType(self):
        return self.attributes.get("scheduleSubType", None)


class SettingsHardwaresettings(LyricBase):
    @property
    def brightness(self):
        return self.attributes.get("brightness", None)

    @property
    def maxBrightness(self):
        return self.attributes.get("maxBrightness", None)


class SettingsTemperaturemode(LyricBase):
    @property
    def air(self):
        return self.attributes.get("air", True)


class SettingsSpecialmode(LyricBase):
    @property
    def _(self):
        return None

class SettingsFan(LyricBase):
    @property
    def fan(self):
        return self.atributes.get("fan", {})
        

class Settings(LyricBase):
    @property
    def hardwareSettings(self):
        return SettingsHardwaresettings(self.attributes.get("hardwareSettings", {}))

    @property
    def temperatureMode(self):
        return SettingsTemperaturemode(self.attributes.get("temperatureMode", {}))

    @property
    def specialMode(self):
        return SettingsSpecialmode(self.attributes.get("specialMode", {}))

    @property
    def devicePairingEnabled(self):
        return self.attributes.get("devicePairingEnabled", True)

    @property
    def fanModes(self):
        return SettingsFan(self.attributes.get("allowedModes", []))

    @property
    def fanChangeableValues(self):
        return SettingsFan(self.attributes.get("changeableValues", {}))
    
    @property
    def fanMode(self):
        return fanChangeableValues(self.attributes.get("mode", None))


class Devicesettings(LyricBase):
    @property
    def _(self):
        return None


class Service(LyricBase):
    @property
    def mode(self):
        return self.attributes.get("mode", None)


class Changeablevalues(LyricBase):
    @property
    def autoChangeoverActive(self):
        return self.attributes.get("autoChangeoverActive", None)

    @property
    def mode(self):
        return self.attributes.get("mode", None)

    @property
    def heatSetpoint(self):
        return self.attributes.get("heatSetpoint", None)

    @property
    def coolSetpoint(self):
        return self.attributes.get("coolSetpoint", None)

    @property
    def thermostatSetpointStatus(self):
        return self.attributes.get("thermostatSetpointStatus", None)

    @property
    def nextPeriodTime(self):
        return self.attributes.get("nextPeriodTime", None)

    @property
    def heatCoolMode(self):
        return self.attributes.get("heatCoolMode", None)

    @property
    def endHeatSetpoint(self):
        return self.attributes.get("endHeatSetpoint", None)

    @property
    def endCoolSetpoint(self):
        return self.attributes.get("endCoolSetpoint", None)


class Operationstatus(LyricBase):
    @property
    def mode(self):
        return self.attributes.get("mode", None)

    @property
    def fanRequest(self):
        return self.attributes.get("fanRequest", False)

    @property
    def circulationFanRequest(self):
        return self.attributes.get("circulationFanRequest", False)


class LyricDevice(LyricBaseClient):
    @property
    def locationID(self):
        return self.attributes.get("locationID", None)

    @property
    def indoorHumidity(self):
        return self.attributes.get("indoorHumidity", None)
    
    @property
    def displayedOutdoorHumidity(self):
        return self.attributes.get("displayedOutdoorHumidity", None)

    @property
    def vacationHold(self):
        return Vacationhold(self.attributes.get("vacationHold", {}))

    @property
    def currentSchedulePeriod(self):
        return Currentscheduleperiod(self.attributes.get("currentSchedulePeriod", {}))

    @property
    def scheduleCapabilities(self):
        return Schedulecapabilities(self.attributes.get("scheduleCapabilities", {}))

    @property
    def scheduleType(self):
        return Scheduletype(self.attributes.get("scheduleType", {}))

    @property
    def scheduleStatus(self):
        return self.attributes.get("scheduleStatus", None)

    @property
    def allowedTimeIncrements(self):
        return self.attributes.get("allowedTimeIncrements", None)

    @property
    def settings(self):
        return Settings(self.attributes.get("settings", {}))

    @property
    def deviceClass(self):
        return self.attributes.get("deviceClass", None)

    @property
    def deviceType(self):
        return self.attributes.get("deviceType", None)

    @property
    def deviceID(self):
        return self.attributes.get("deviceID", None)

    @property
    def name(self):
        return self.attributes.get("name", None)

    @property
    def isAlive(self):
        return self.attributes.get("isAlive", True)

    @property
    def isUpgrading(self):
        return self.attributes.get("isUpgrading", False)

    @property
    def isProvisioned(self):
        return self.attributes.get("isProvisioned", True)

    @property
    def macID(self):
        return self.attributes.get("macID", None)

    @property
    def deviceSettings(self):
        return Devicesettings(self.attributes.get("deviceSettings", {}))

    @property
    def service(self):
        return Service(self.attributes.get("service", {}))

    @property
    def deviceRegistrationDate(self):
        return self.attributes.get("deviceRegistrationDate", None)

    @property
    def dataSyncStatus(self):
        return self.attributes.get("dataSyncStatus", None)

    @property
    def units(self):
        return self.attributes.get("units", None)

    @property
    def indoorTemperature(self):
        return self.attributes.get("indoorTemperature", None)

    @property
    def outdoorTemperature(self):
        return self.attributes.get("outdoorTemperature", None)

    @property
    def allowedModes(self):
        return self.attributes.get("allowedModes", [])

    @property
    def deadband(self):
        return self.attributes.get("deadband", None)

    @property
    def hasDualSetpointStatus(self):
        return self.attributes.get("hasDualSetpointStatus", False)

    @property
    def minHeatSetpoint(self):
        return self.attributes.get("minHeatSetpoint", None)

    @property
    def maxHeatSetpoint(self):
        return self.attributes.get("maxHeatSetpoint", None)

    @property
    def minCoolSetpoint(self):
        return self.attributes.get("minCoolSetpoint", None)

    @property
    def maxCoolSetpoint(self):
        return self.attributes.get("maxCoolSetpoint", None)

    @property
    def changeableValues(self):
        return Changeablevalues(self.attributes.get("changeableValues", {}))

    @property
    def operationStatus(self):
        return Operationstatus(self.attributes.get("operationStatus", {}))

    @property
    def deviceModel(self):
        return self.attributes.get("deviceModel", None)



"""
Class object for LyricPriority
Documentation: https://github.com/timmo001/aiolyric

Generated by generator/generator.py - 2023-07-27 18:30:29.139453
"""
from .base import LyricBase


class Accessories(LyricBase):

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def excludeTemp(self):
        return self.attributes.get("excludeTemp", False)

    @property
    def excludeMotion(self):
        return self.attributes.get("excludeMotion", False)

    @property
    def temperature(self):
        return self.attributes.get("temperature", None)

    @property
    def status(self):
        return self.attributes.get("status", "")

    @property
    def detectMotion(self):
        return self.attributes.get("detectMotion", False)


class Rooms(LyricBase):

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def roomName(self):
        return self.attributes.get("roomName", "")

    @property
    def roomAvgTemp(self):
        return self.attributes.get("roomAvgTemp", None)

    @property
    def roomAvgHumidity(self):
        return self.attributes.get("roomAvgHumidity", None)

    @property
    def overallMotion(self):
        return self.attributes.get("overallMotion", False)

    @property
    def accessories(self):
        return [Accessories(x) for x in self.attributes.get("accessories", [])]


class Currentpriority(LyricBase):

    @property
    def priorityType(self):
        return self.attributes.get("priorityType", "")

    @property
    def selectedRooms(self):
        return self.attributes.get("selectedRooms", [])

    @property
    def rooms(self):
        return [Rooms(x) for x in self.attributes.get("rooms", [])]


class LyricPriority(LyricBase):

    @property
    def deviceId(self):
        return self.attributes.get("deviceId", "")

    @property
    def status(self):
        return self.attributes.get("status", "")

    @property
    def currentPriority(self):
        return Currentpriority(self.attributes.get("currentPriority", {}))





"""Lyric: Client"""
import async_timeout
import logging

from abc import abstractmethod
from asyncio import CancelledError, TimeoutError, get_event_loop
from aiohttp import ClientError, ClientSession, ClientResponse

from ..objects.base import LyricBase
from ..exceptions import LyricException, LyricAuthenticationException


class LyricClient(LyricBase):
    """Client to handle API calls."""

    def __init__(self, session: ClientSession) -> None:
        """Initialize the client."""
        self._session = session

    @abstractmethod
    async def async_get_access_token(self) -> str:
        """Return a valid access token."""

    async def get(self, url: str, **kwargs) -> ClientResponse:
        """Make a GET request."""
        return await self.request("GET", url, **kwargs)

    async def post(self, url: str, **kwargs) -> ClientResponse:
        """Make a POST request."""
        return await self.request("POST", url, **kwargs)

    async def request(
        self, method: str in ["GET", "POST"], url: str, **kwargs
    ) -> ClientResponse:
        """Make a request."""
        headers = kwargs.get("headers")

        if headers is None:
            headers = {}
        else:
            headers = dict(headers)

        access_token = await self.async_get_access_token()
        headers["Authorization"] = f"Bearer {access_token}"
        headers["Content-Type"] = "application/json"

        async with async_timeout.timeout(20):
            response: ClientResponse = await self._session.request(
                method,
                url,
                headers=headers,
                **kwargs,
            )
        if response.status != 200:
            if response.status == 401:
                raise LyricAuthenticationException(
                    {
                        "request": {
                            "method": method,
                            "url": url,
                            "headers": headers,
                            **kwargs,
                        },
                        "response": await response.json(),
                        "status": response.status,
                    }
                )
            else:
                raise LyricException(
                    {
                        "request": {
                            "method": method,
                            "url": url,
                            "headers": headers,
                            **kwargs,
                        },
                        "response": await response.json(),
                        "status": response.status,
                    }
                )
        return response



"""Lyric: Init"""
from typing import List

from aiohttp import ClientResponse

from .const import BASE_URL
from .objects.base import LyricBase
from .objects.device import LyricDevice
from .objects.location import LyricLocation
from .objects.priority import LyricPriority


class Lyric(LyricBase):
    """Handles authentication refresh tokens."""

    def __init__(
        self,
        client: "LyricClient",
        client_id: str,
    ) -> None:
        """Initialize the token manager class."""
        self._client = client
        self._client_id = client_id
        self._devices: List[LyricDevice] = []
        self._devices_dict: dict = {}
        self._locations: List[LyricLocation] = []
        self._locations_dict: dict = {}
        self._rooms: List

    @property
    def client_id(self) -> str:
        return self._client_id

    @property
    def devices(self) -> List[LyricDevice]:
        return self._devices

    @property
    def devices_dict(self) -> dict:
        return self._devices_dict

    @property
    def locations(self) -> List[LyricLocation]:
        return self._locations

    @property
    def locations_dict(self) -> dict:
        return self._locations_dict

    async def get_devices(
        self,
        location_id: int,
    ) -> None:
        """Get Devices."""
        response: ClientResponse = await self._client.get(
            f"{BASE_URL}/devices?apikey={self.client_id}&locationId={location_id}"
        )
        json = await response.json()
        self.logger.debug(json)
        self._devices = [LyricDevice(self._client, device) for device in json or []]
        self._devices_dict: dict = {}
        for device in self._devices:
            self._devices_dict[device.macID] = device

    async def get_locations(self) -> None:
        """Get Locations."""
        response: ClientResponse = await self._client.get(
            f"{BASE_URL}/locations?apikey={self.client_id}"
        )
        json = await response.json()
        self.logger.debug(json)
        self._locations = [
            LyricLocation(self._client, location) for location in json or []
        ]
        self._locations_dict: dict = {}
        for location in self._locations:
            self._locations_dict[location.locationID] = location

    async def get_priority_and_rooms(
            self,
            location_id: int
        ) -> None:
        """Get Priority, which contains accessory information."""
        response: ClientResponse = await self._client.get(
            f"{BASE_URL}/priority?apikey={self.client_id}&locationId={location_id}"
        )
        json = await response.json()
        self.logger.debug(json)
        
        self._priority = LyricPriority(self._client, json)
        self._rooms = self._priority.currentPriority.rooms    

    async def update_thermostat(
        self,
        location: LyricLocation,
        device: LyricDevice,
        mode=None,
        heatSetpoint=None,
        coolSetpoint=None,
        autoChangeoverActive=None,
        thermostatSetpointStatus=None,
        nextPeriodTime=None,
    ) -> ClientResponse:
        """Update Theremostat."""
        self.logger.debug("Update Thermostat")

        data = {}

        if mode is not None:
            data["mode"] = mode
        else:
            data["mode"] = device.changeableValues.mode

        if heatSetpoint is not None:
            data["heatSetpoint"] = heatSetpoint
        else:
            data["heatSetpoint"] = device.changeableValues.heatSetpoint
        if coolSetpoint is not None:
            data["coolSetpoint"] = coolSetpoint
        else:
            data["coolSetpoint"] = device.changeableValues.coolSetpoint

        # Only for TCC devices
        if autoChangeoverActive is not None:
            data["autoChangeoverActive"] = autoChangeoverActive
        elif device.changeableValues.autoChangeoverActive is not None:
            data["autoChangeoverActive"] = device.changeableValues.autoChangeoverActive

        # Only for LCC devices
        if thermostatSetpointStatus is not None:
            data["thermostatSetpointStatus"] = thermostatSetpointStatus
        elif device.changeableValues.thermostatSetpointStatus is not None:
            if device.changeableValues.thermostatSetpointStatus == "NoHold":
                data["thermostatSetpointStatus"] = "TemporaryHold"
            else:
                data[
                    "thermostatSetpointStatus"
                ] = device.changeableValues.thermostatSetpointStatus

        if data.get("thermostatSetpointStatus", "") == "HoldUntil":
            if nextPeriodTime is not None:
                data["nextPeriodTime"] = nextPeriodTime
            elif device.changeableValues.nextPeriodTime == "NoHold" and mode is None:
                data["nextPeriodTime"] = "TemporaryHold"
            else:
                data["nextPeriodTime"] = device.changeableValues.nextPeriodTime

        self.logger.debug(data)

        return await self._client.post(
            f"{BASE_URL}/devices/thermostats/{device.deviceID}?apikey={self._client_id}&locationId={location.locationID}",
            json=data,
        )

    async def update_fan(
        self,
        location: LyricLocation,
        device: LyricDevice,
        mode: str,
    ) -> ClientResponse:
        """Update Fan."""
        self.logger.debug("Update Fan")

        data = {}

        if mode is not None:
            data["mode"] = mode
        else:
            data["mode"] = device.fanMode

        self.logger.debug(data)

        return await self._client.post(
            f"{BASE_URL}/devices/thermostats/{device.deviceID}/fan?apikey={self._client_id}&locationId={location.locationID}",
            json=data,
        )



#!/usr/bin/env python
import io

from setuptools import find_packages, setup

version = "1.1.0"

setup(
    name="dalinicus.aiolyric",
    version=version,
    description="AIO package for the Honeywell Lyric Platform.",
    long_description=io.open("README.md", encoding="UTF-8").read(),
    keywords="honeywell lyric thermostat",
    author="Aidan Timson (Timmo)",
    author_email="contact@timmo.xyz",
    license="MIT",
    url="https://github.com/timmo001/aiolyric",
    packages=find_packages(exclude=["tests", "generator"]),
    install_requires=["aiohttp>=3.7.3"],
)



"""Lyric: Base"""
import logging


class LyricBase:
    """Base class for Lyric."""

    logger = logging.getLogger(__name__)

    def __init__(self, attributes) -> None:
        """Initialize."""
        self.attributes = attributes


class LyricBaseClient(LyricBase):
    """Base class for Lyric."""

    def __init__(self, client: "AIOGitHubAPIClient", attributes: dict) -> None:
        """Initialise."""
        super().__init__(attributes)
        self.client = client